@@ -1,7 +1,7 @@
 use strict;
 use warnings;
 package Time::C;
-$Time::C::VERSION = '0.020';
+$Time::C::VERSION = '0.021';
 # ABSTRACT: Convenient time manipulation.
 
 use overload (
@@ -21,10 +21,32 @@ use Time::Zone::Olson;
 
 
 
-method new ($c: $year = undef, $month = 1, $day = 1, $hour = 0, $minute = 0, $second = 0, $tz = 'UTC') {
-    return $c->now_utc() if not defined $year;
+method new ($c: $year =, $month =, $day =, $hour =, $minute =, $second =, $tz =) {
+    my $t;
+    my %defineds = (
+        epoch_d => (defined($year) and defined($month) and defined($day) and defined($hour) and defined($minute) and defined($second)),
+        year => defined($year),
+        month => defined($month),
+        mday => defined($day),
+        week => 0,
+        wday => 0,
+        yday => 0,
+        hour => defined($hour),
+        minute => defined($minute),
+        second => defined($second),
+        tz_d => defined($tz),
+        offset => 0,
+    );
 
-    my $tm = Time::Moment->new(year => $year, month => $month, day => $day, hour => $hour, minute => $minute, second => $second, offset => 0);
+    if (not defined $year) {
+        $t = $c->now_utc();
+        %$t = ( %$t, %defineds );
+        return $t;
+    }
+
+    my $tm = Time::Moment->new(year => $year, month => $month // 1, day => $day // 1, hour => $hour // 0, minute => $minute // 0, second => $second // 0, offset => 0);
+
+    if (not defined $tz) { $tz = 'UTC'; }
 
     if ($tz ne 'UTC' and $tz ne 'GMT') {
         my $offset = _get_offset($tm->epoch, $tz);
@@ -33,7 +55,9 @@ method new ($c: $year = undef, $month = 1, $day = 1, $hour = 0, $minute = 0, $se
         $tm = $tm->with_offset_same_local($offset);
     }
 
-    $c->localtime($tm->epoch, $tz);
+    $t = $c->localtime($tm->epoch, $tz);
+    %$t = (%$t, %defineds);
+    return $t;
 }
 
 
@@ -44,6 +68,20 @@ method mktime ($c: :$epoch =, :$second =, :$minute =, :$hour =, :$mday =, :$mont
     # Then go on to creating it from the date bits, and the time bits
 
     my $t;
+    my %defineds = (
+        epoch_d => defined($epoch),
+        year => defined($year),
+        month => defined($month),
+        mday => defined($mday),
+        week => defined($week),
+        wday => defined($wday),
+        yday => defined($yday),
+        hour => defined($hour),
+        minute => defined($minute),
+        second => defined($second),
+        tz_d => defined($tz),
+        offset => defined($offset),
+    );
 
     if (defined $epoch) {
         $t = Time::C->gmtime($epoch);
@@ -54,6 +92,8 @@ method mktime ($c: :$epoch =, :$second =, :$minute =, :$hour =, :$mday =, :$mont
             $t->offset = $offset;
         }
 
+        $t->{tz_d} = $defineds{tz_d};
+
         return $t;
     } elsif (defined $year) { # We have a year at least...
         if (defined $month) {
@@ -64,10 +104,11 @@ method mktime ($c: :$epoch =, :$second =, :$minute =, :$hour =, :$mday =, :$mont
             }
         } elsif (defined $week) {
             $t = Time::C->new($year);
-            $t->day++ while ($t->week > 1);
+            $t->day_of_week++ while ($t->week > 1);
             $t = $t->week($week)->day_of_week(1);
 
             if (defined $wday) { $t->day_of_week = $wday; }
+            else { $t->{wday} = 0; }
         } elsif (defined $yday) {
             $t = Time::C->new($year)->day($yday);
         } else { # we have neither month, week, or day of year!
@@ -94,9 +135,11 @@ method mktime ($c: :$epoch =, :$second =, :$minute =, :$hour =, :$mday =, :$mont
             if (defined $second) { $t->second = $second; }
         } elsif (defined $week) {
             $t = Time::C->new($year);
-            $t->day++ while ($t->week > 1);
+            $t->day_of_week++ while ($t->week > 1);
             $t = $t->week($week)->day_of_week(1);
+
             if (defined $wday) { $t->day_of_week = $wday; }
+            else { $t->{wday} = 0; }
 
             # Now add the time bits on top...
             if (defined $hour) { $t->hour = $hour; }
@@ -124,6 +167,7 @@ method mktime ($c: :$epoch =, :$second =, :$minute =, :$hour =, :$mday =, :$mont
             if (defined $minute) { $t->minute = $minute; } elsif (not defined $hour) { $t->second_of_day = Time::C->now($tz)->tz('UTC', 1)->second(0)->second_of_day; }
             if (defined $second) { $t->second = $second; }
         }
+        $t->{year} = 0;
     }
 
     # And last, adjust for timezone bits
@@ -131,7 +175,7 @@ method mktime ($c: :$epoch =, :$second =, :$minute =, :$hour =, :$mday =, :$mont
     if (defined $tz) {
         $t = $t->tz($tz, 1);
     } elsif (defined $offset) {
-        $t->tm = $t->tm->with_offset_same_local($offset);
+        $t->_tm = $t->tm->with_offset_same_local($offset);
         $t->offset = $offset;
     }
 
@@ -140,9 +184,23 @@ method mktime ($c: :$epoch =, :$second =, :$minute =, :$hour =, :$mday =, :$mont
 
 
 method localtime ($c: $epoch, $tz = $ENV{TZ}) {
+    my %defineds = (
+        epoch_d => 1,
+        year => 1,
+        month => 1,
+        mday => 1,
+        week => 1,
+        wday => 1,
+        yday => 1,
+        hour => 1,
+        minute => 1,
+        second => 1,
+        tz_d => defined($tz),
+        offset => 0,
+    );
     $tz = 'UTC' unless defined $tz;
     _verify_tz($tz);
-    bless {epoch => $epoch, tz => $tz}, $c;
+    bless {epoch => $epoch, tz => $tz, %defineds}, $c;
 }
 
 
@@ -156,14 +214,62 @@ method now_utc ($c:) { $c->localtime( time, 'UTC' ); }
 
 
 method from_string ($c: $str, :$format = undef, :$locale = 'C', :$strict = 1, :$tz = 'UTC') {
-    my @p = _parse($str, $format, $tz, locale => $locale, strict => $strict);
+    my %p = _parse($str, $format, $tz, locale => $locale, strict => $strict);
 
-    bless {epoch => $p[0], tz => $p[1]}, $c;
+    return $c->mktime(%p);
 }
 
 
 method strptime ($c: $str, $format, :$locale = 'C', :$strict = 1) {
-    return Time::P::strptime(@_);
+    my %struct;
+    if (ref $c) {
+        my $t = $c;
+        if ($t->{epoch_d}) { $struct{epoch} = $t->{epoch}; }
+        if ($t->{year}) { $struct{year} = $t->year; }
+        if ($t->{month}) { $struct{month} = $t->month; }
+        if ($t->{mday}) { $struct{mday} = $t->day; }
+        if ($t->{week}) { $struct{week} = $t->week; }
+        if ($t->{wday}) { $struct{wday} = $t->day_of_week; }
+        if ($t->{yday}) { $struct{yday} = $t->day_of_year; }
+        if ($t->{hour}) { $struct{hour} = $t->hour; }
+        if ($t->{minute}) { $struct{minute} = $t->minute; }
+        if ($t->{second}) { $struct{second} = $t->second; }
+        if ($t->{tz_d}) { $struct{tz} = $t->{tz}; }
+        if ($t->{offset}) { $struct{offset} = $t->offset; }
+    }
+    %struct = Time::P::strptime($str, $format, locale => $locale, strict => $strict, struct => \%struct);
+
+    if (ref $c) {
+        my $t = $c;
+
+        if (defined $struct{tz}) {
+            $t->tz = $struct{tz};
+        } elsif (defined $struct{offset}) {
+            $t->offset = $struct{offset};
+        }
+
+        if (defined $struct{epoch}) { return $t->epoch($struct{epoch}); }
+
+        if (defined $struct{year}) { $t->year = $struct{year}; }
+
+        if (defined $struct{month}) {
+            $t->month = $struct{month};
+            if (defined $struct{mday}) { $t->day = $struct{mday}; }
+        } elsif (defined $struct{week}) {
+            $t->week = $struct{week};
+            if (defined $struct{wday}) { $t->day_of_week = $struct{wday}; }
+        } elsif (defined $struct{yday}) {
+            $t->day_of_year = $struct{yday};
+        }
+
+        if (defined $struct{hour}) { $t->hour = $struct{hour}; }
+        if (defined $struct{minute}) { $t->minute = $struct{minute}; }
+        if (defined $struct{second}) { $t->second = $struct{second}; }
+
+        return $t;
+    } else {
+        return $c->mktime(%struct);
+    }
 }
 
 fun _verify_tz ($tz) {
@@ -215,8 +321,9 @@ fun _get_tz ($offset) {
 
 fun _parse ($str, $format = undef, $tz = 'UTC', :$locale = 'C', :$strict = 1) {
     if (defined $format) {
-        my $t = eval { Time::P::strptime($str, $format, locale => $locale, strict => $strict) };
-        return ($t->epoch, $t->tz) if defined $t;
+        my %struct = ();
+        my $e = eval { %struct = Time::P::strptime($str, $format, locale => $locale, strict => $strict); 1; };
+        return %struct if $e;
 
         croak sprintf "Could not parse %s using %s: %s", $str, $format, $@;
     }
@@ -227,10 +334,12 @@ fun _parse ($str, $format = undef, $tz = 'UTC', :$locale = 'C', :$strict = 1) {
     my $epoch = $tm->epoch;
     my $offset = $tm->offset;
 
-    $tz = _get_tz($offset) unless $offset ==
-      Time::Zone::Olson->new({timezone => $tz})->local_offset($epoch);
+    my @ret = (epoch => $epoch, offset => $offset);
+    if ($offset == Time::Zone::Olson->new({timezone => $tz})->local_offset($epoch)) {
+        push @ret, tz => $tz;
+    }
 
-    return $epoch, $tz;
+    return @ret;
 }
 
 
@@ -240,6 +349,9 @@ method epoch ($t: $new_epoch = undef) :lvalue {
 
     my $setter = sub {
         $t->{epoch} = $_[0];
+
+        %$t = (%$t, epoch_d => 1, year => 1, yday => 1, month => 1, mday => 1, week => 1, wday => 1, hour => 1,  minute => 1, second => 1, );
+
         return $t if defined $new_epoch;
         return $_[0];
     };
@@ -256,10 +368,12 @@ method tz ($t: $new_tz = undef, $override = 0) :lvalue {
 
         if ($override) {
             my $l = Time::C->new($t->year, $t->month, $t->day, $t->hour, $t->minute, $t->second, $_[0]);
-            $t->epoch = $l->epoch;
+            $t->{epoch} = $l->epoch;
         }
 
         $t->{tz} = $_[0];
+        $t->{tz_d} = 1;
+
         return $t if defined $new_tz;
         return $t->{tz};
     };
@@ -291,6 +405,7 @@ fun _get_offset ($epoch, $tz) {
 method offset ($t: $new_offset = undef) :lvalue {
     my $setter = sub {
         $t->{tz} = _get_tz($_[0]);
+        $t->{offset} = 1;
 
         return $t if defined $new_offset;
         return $_[0];
@@ -305,6 +420,20 @@ method offset ($t: $new_offset = undef) :lvalue {
 
 
 method tm ($t: $new_tm = undef) :lvalue {
+    my $setter = sub {
+        $t->_tm = $_[0];
+        $t->epoch = $t->epoch; # update definedness values
+
+        return $t if defined $new_tm;
+        return $_[0];
+    };
+
+    return $setter->($new_tm) if defined $new_tm;
+
+    sentinel value => $t->_tm(), set => $setter;
+}
+
+method _tm ($t: $new_tm = undef) :lvalue {
     $t->{tz} = 'UTC' if not defined $t->{tz};
 
     my $setter = sub {
@@ -330,7 +459,31 @@ method string ($t: $new_str = undef, :$format = undef, :$locale = 'C', :$strict
     $t->{tz} = 'UTC' if not defined $t->{tz};
 
     my $setter = sub {
-        @{$t}{epoch,tz} = _parse($_[0], $format, $t->{tz}, locale => $locale, strict => $strict);
+        my %struct = _parse($_[0], $format, $t->{tz}, locale => $locale, strict => $strict);
+
+        if (defined $struct{tz}) {
+            $t->tz = $struct{tz};
+        } elsif (defined $struct{offset}) {
+            $t->offset = $struct{offset};
+        }
+
+        if (defined $struct{epoch}) { return $t->epoch($struct{epoch}); }
+
+        if (defined $struct{year}) { $t->year = $struct{year}; }
+
+        if (defined $struct{month}) {
+            $t->month = $struct{month};
+            if (defined $struct{mday}) { $t->day = $struct{mday}; }
+        } elsif (defined $struct{week}) {
+            $t->week = $struct{week};
+            if (defined $struct{wday}) { $t->day_of_week = $struct{wday}; }
+        } elsif (defined $struct{yday}) {
+            $t->day_of_year = $struct{yday};
+        }
+
+        if (defined $struct{hour}) { $t->hour = $struct{hour}; }
+        if (defined $struct{minute}) { $t->minute = $struct{minute}; }
+        if (defined $struct{second}) { $t->second = $struct{second}; }
 
         return $t if defined $new_str;
         return $_[0];
@@ -356,7 +509,8 @@ method year ($t: $new_year = undef) :lvalue {
     my $tm = $t->tm();
 
     my $setter = sub {
-        my $ret = ($t->tm = $tm->with_year($_[0]))->year;
+        my $ret = ($t->_tm = $tm->with_year($_[0]))->year;
+        $t->{year} = 1;
 
         return $t if defined $new_year;
         return $ret;
@@ -372,7 +526,8 @@ method quarter ($t: $new_quarter = undef) :lvalue {
     my $tm = $t->tm();
 
     my $setter = sub {
-        my $ret = ($t->tm = $tm->plus_months(3*$_[0] - $tm->month))->quarter;
+        my $ret = ($t->_tm = $tm->plus_months(3*$_[0] - $tm->month))->quarter;
+        $t->{month} = 1;
 
         return $t if defined $new_quarter;
         return $ret;
@@ -388,7 +543,8 @@ method month ($t: $new_month = undef) :lvalue {
     my $tm = $t->tm();
 
     my $setter = sub {
-        my $ret = ($t->tm = $tm->plus_months($_[0] - $tm->month))->month;
+        my $ret = ($t->_tm = $tm->plus_months($_[0] - $tm->month))->month;
+        $t->{month} = 1;
 
         return $t if defined $new_month;
         return $ret;
@@ -404,7 +560,8 @@ method week ($t: $new_week = undef) :lvalue {
     my $tm = $t->tm();
 
     my $setter = sub {
-        my $ret = ($t->tm = $tm->plus_weeks($_[0] - $tm->week))->week;
+        my $ret = ($t->_tm = $tm->plus_weeks($_[0] - $tm->week))->week;
+        $t->{week} = 1;
 
         return $t if defined $new_week;
         return $ret;
@@ -423,7 +580,8 @@ method day_of_month ($t: $new_day = undef) :lvalue {
     my $tm = $t->tm();
 
     my $setter = sub {
-        my $ret = ($t->tm = $tm->plus_days($_[0] - $tm->day_of_month))->day_of_month;
+        my $ret = ($t->_tm = $tm->plus_days($_[0] - $tm->day_of_month))->day_of_month;
+        $t->{mday} = 1;
 
         return $t if defined $new_day;
         return $ret;
@@ -439,7 +597,8 @@ method day_of_year ($t: $new_day = undef) :lvalue {
     my $tm = $t->tm();
 
     my $setter = sub {
-        my $ret = ($t->tm = $tm->plus_days($_[0] - $tm->day_of_year))->day_of_year;
+        my $ret = ($t->_tm = $tm->plus_days($_[0] - $tm->day_of_year))->day_of_year;
+        $t->{yday} = 1;
 
         return $t if defined $new_day;
         return $ret;
@@ -455,7 +614,8 @@ method day_of_quarter ($t: $new_day = undef) :lvalue {
     my $tm = $t->tm();
 
     my $setter = sub {
-        my $ret = ($t->tm = $tm->plus_days($_[0] - $tm->day_of_quarter))->day_of_quarter;
+        my $ret = ($t->_tm = $tm->plus_days($_[0] - $tm->day_of_quarter))->day_of_quarter;
+        $t->{mday} = 1;
 
         return $t if defined $new_day;
         return $ret;
@@ -471,7 +631,8 @@ method day_of_week ($t: $new_day = undef) :lvalue {
     my $tm = $t->tm();
 
     my $setter = sub {
-        my $ret = ($t->tm = $tm->plus_days($_[0] - $tm->day_of_week))->day_of_week;
+        my $ret = ($t->_tm = $tm->plus_days($_[0] - $tm->day_of_week))->day_of_week;
+        $t->{wday} = 1;
 
         return $t if defined $new_day;
         return $ret;
@@ -487,7 +648,8 @@ method hour ($t: $new_hour = undef) :lvalue {
     my $tm = $t->tm();
 
     my $setter = sub {
-        my $ret = ($t->tm = $tm->plus_hours($_[0] - $tm->hour))->hour;
+        my $ret = ($t->_tm = $tm->plus_hours($_[0] - $tm->hour))->hour;
+        $t->{hour} = 1;
 
         return $t if defined $new_hour;
         return $ret;
@@ -503,7 +665,8 @@ method minute ($t: $new_minute = undef) :lvalue {
     my $tm = $t->tm();
 
     my $setter = sub {
-        my $ret = ($t->tm = $tm->plus_minutes($_[0] - $tm->minute))->minute;
+        my $ret = ($t->_tm = $tm->plus_minutes($_[0] - $tm->minute))->minute;
+        $t->{minute} = 1;
 
         return $t if defined $new_minute;
         return $ret;
@@ -519,7 +682,8 @@ method second ($t: $new_second = undef) :lvalue {
     my $tm = $t->tm();
 
     my $setter = sub {
-        my $ret = ($t->tm = $tm->plus_seconds($_[0] - $tm->second))->second;
+        my $ret = ($t->_tm = $tm->plus_seconds($_[0] - $tm->second))->second;
+        $t->{second} = 1;
 
         return $t if defined $new_second;
         return $ret;
@@ -535,7 +699,8 @@ method second_of_day ($t: $new_second = undef) :lvalue {
     my $tm = $t->tm();
 
     my $setter = sub {
-        my $ret = ($t->tm = $tm->plus_seconds($_[0] - $tm->second_of_day))->second_of_day;
+        my $ret = ($t->_tm = $tm->plus_seconds($_[0] - $tm->second_of_day))->second_of_day;
+        $t->{second} = $t->{minute} = $t->{hour} = 1;
 
         return $t if defined $new_second;
         return $ret;
@@ -562,7 +727,8 @@ method diff ($t: $t2) {
 method clone($t:) {
     my $c = ref $t;
 
-    return $c->localtime($t->epoch, $t->tz);
+    my $t2 = { %$t };
+    bless $t2, $c;
 }
 
 1;
@@ -579,7 +745,7 @@ Time::C - Convenient time manipulation.
 
 =head1 VERSION
 
-version 0.020
+version 0.021
 
 =head1 SYNOPSIS
 
@@ -657,7 +823,7 @@ This is the timezone specification such as C<Europe/Stockholm> or C<UTC>. If not
 =head2 mktime
 
   my $t = Time::C->mktime(
-    epoch => epoch,
+    epoch => $epoch,
     second => $second,
     minute => $minute,
     hour => $hour,
@@ -821,8 +987,14 @@ If there is no valid timezone specified in the format, but C<$tz> is given and m
   my $t = Time::C->strptime($str, $format, locale => $locale);
   my $t = Time::C->strptime($str, $format, locale => $locale, strict => $strict);
 
+  $t = $t->strptime($str, $format);
+  $t = $t->strptime($str, $format, locale => $locale);
+  $t = $t->strptime($str, $format, locale => $locale, strict => $strict);
+
 Creates a Time::C object for the specified C<$str> using the C<$format> to parse it with L<Time::P/strptime>.
 
+This doesn't need to be used solely as a constructor; if it's called on an already existing C<Time::C> object, the values parsed from the C<$str> will be updated in the object, following the same rules as C<< Time::C->mktime >> for precedence (i.e. if an epoch is supplied, none of the other values matter, and if a month is supplied, the weeks and weekdays won't be considered, and so on).
+
 =over
 
 =item C<$str>
